class ShareVerifier:
    def __init__(self, p: int, g: int):
        """
        Инициализация верификатора
        :param p: простое число (должно совпадать с генератором)
        :param g: генератор группы
        """
        self.p = p
        self.g = g
    
    def verify_share(self, share: Tuple[int, int], commitments: List[int]) -> bool:
        """
        Проверка корректности доли
        :param share: кортеж (id, значение доли)
        :param commitments: список обязательств
        :return: True если доля корректна
        """
        x, y = share
        if not commitments:
            raise ValueError("Необходимы обязательства для проверки")
            
        # Вычисление g^y mod p
        lhs = pow(self.g, y, self.p)
        
        # Вычисление произведения commitments[j]^(x^j) mod p
        rhs = 1
        for j, comm in enumerate(commitments):
            rhs = (rhs * pow(comm, x ** j, self.p)) % self.p
            
        return lhs == rhs


class SecretReconstructor:
    def __init__(self, p: int):
        """
        Инициализация реконструктора
        :param p: простое число (модуль)
        """
        self.p = p
    
    def reconstruct_secret(self, shares: List[Tuple[int, int]]) -> int:
        """
        Восстановление секрета по долям
        :param shares: список долей [(x1, y1), (x2, y2), ...]
        :return: восстановленный секрет
        """
        if not shares:
            raise ValueError("Необходимы доли для восстановления")
            
        secret = 0
        for i, (xi, yi) in enumerate(shares):
            numerator, denominator = 1, 1
            
            for j, (xj, _) in enumerate(shares):
                if i != j:
                    numerator *= (-xj)
                    denominator *= (xi - xj)
                    
            lagrange_coeff = (numerator * pow(denominator, -1, self.p)) % self.p
            secret = (secret + yi * lagrange_coeff) % self.p
            
        return secret


from .utills import isprime, primitive_root
from .secret_reconstructor import SecretReconstructor
from .share_generator import ShareGenerator
from .share_verifier import ShareVerifier
from .testf import crypto_params, setup_components, TestUtils, TestShareGenerator, TestShareVerifier, TestSecretReconstructor, test_full_workflow


import random
from typing import List, Tuple

class ShareGenerator:
    def __init__(self, p: int, g: int):
        """
        Инициализация с параметрами схемы
        :param p: простое число
        :param g: генератор группы
        """
        self.p = p
        self.g = g
    
    def generate_shares(self, secret: int, n: int, t: int) -> Tuple[List[Tuple[int, int]], List[int]]:
        """
        Генерация долей секрета
        :param secret: секретное число
        :param n: общее количество долей
        :param t: пороговое количество
        :return: (доли, коэффициенты полинома)
        """
        if not 0 < secret < self.p:
            raise ValueError("Секрет должен быть в диапазоне (0, p)")
        if t > n:
            raise ValueError("Порог t не может быть больше n")
        
        coefficients = [secret] + [random.randint(1, self.p-1) for _ in range(t-1)]
        shares = []
        
        for i in range(1, n+1):
            share = sum(coeff * (i ** j) for j, coeff in enumerate(coefficients)) % self.p
            shares.append((i, share))
            
        return shares, coefficients
    
    def generate_commitments(self, coefficients: List[int]) -> List[int]:
        """
        Генерация обязательств Фельдмана
        :param coefficients: коэффициенты полинома
        :return: список обязательств [g^a0, g^a1, ...]
        """
        return [pow(self.g, coeff, self.p) for coeff in coefficients]


import random
from sympy import isprime, primitive_root

def generate_prime(bits: int = 256) -> int:
    """Генерация простого числа заданной длины"""
    while True:
        p = random.getrandbits(bits)
        if isprime(p):
            return p

def find_generator(p: int) -> int:
    """Поиск генератора для простого числа"""
    if not isprime(p):
        raise ValueError("Число должно быть простым")
    return primitive_root(p)


import pytest
from share_generator import ShareGenerator
from share_verifier import ShareVerifier
from secret_reconstructor import SecretReconstructor
from utils import generate_prime, find_generator

# Фикстуры для тестов
@pytest.fixture
def crypto_params():
    p = generate_prime(256)  # Используем 256-битное простое для тестов
    g = find_generator(p)
    return p, g

@pytest.fixture
def setup_components(crypto_params):
    p, g = crypto_params
    generator = ShareGenerator(p, g)
    verifier = ShareVerifier(p, g)
    reconstructor = SecretReconstructor(p)
    return generator, verifier, reconstructor, p, g

# Тесты для генерации простых чисел и генераторов
class TestUtils:
    def test_generate_prime(self):
        p = generate_prime(256)
        assert p > 0
        assert p.bit_length() >= 256

    def test_find_generator(self, crypto_params):
        p, g = crypto_params
        seen = set()
        for i in range(1, p):
            val = pow(g, i, p)
            assert val not in seen
            seen.add(val)
            if len(seen) == p - 1:
                break

    def test_find_generator_with_non_prime(self):
        with pytest.raises(ValueError):
            find_generator(100)  # 100 - не простое число

# Тесты для генератора долей
class TestShareGenerator:
    def test_generate_shares_valid(self, setup_components):
        gen, _, _, p, _ = setup_components
        secret = 12345
        n, t = 5, 3
        
        shares, coeffs = gen.generate_shares(secret, n, t)
        
        assert len(shares) == n
        assert len(coeffs) == t
        assert coeffs[0] == secret

    def test_generate_shares_invalid_secret(self, setup_components):
        gen, _, _, p, _ = setup_components
        with pytest.raises(ValueError):
            gen.generate_shares(p + 1, 5, 3)

    def test_generate_shares_invalid_threshold(self, setup_components):
        gen, _, _, _, _ = setup_components
        with pytest.raises(ValueError):
            gen.generate_shares(12345, 3, 5)

    def test_generate_commitments(self, setup_components):
        gen, _, _, p, g = setup_components
        coeffs = [123, 456, 789]
        commitments = gen.generate_commitments(coeffs)
        
        assert len(commitments) == len(coeffs)
        for comm, coeff in zip(commitments, coeffs):
            assert pow(g, coeff, p) == comm

# Тесты для верификатора долей
class TestShareVerifier:
    def test_verify_valid_share(self, setup_components):
        gen, ver, _, _, _ = setup_components
        shares, coeffs = gen.generate_shares(12345, 5, 3)
        commitments = gen.generate_commitments(coeffs)
        
        for share in shares:
            assert ver.verify_share(share, commitments)

    def test_verify_tampered_share(self, setup_components):
        gen, ver, _, _, _ = setup_components
        shares, coeffs = gen.generate_shares(12345, 5, 3)
        commitments = gen.generate_commitments(coeffs)
        
        tampered_share = (shares[0][0], shares[0][1] + 1)
        assert not ver.verify_share(tampered_share, commitments)

    def test_verify_empty_commitments(self, setup_components):
        _, ver, _, _, _ = setup_components
        with pytest.raises(ValueError):
            ver.verify_share((1, 12345), [])

# Тесты для реконструктора секрета
class TestSecretReconstructor:
    def test_reconstruct_secret(self, setup_components):
        gen, _, recon, _, _ = setup_components
        secret = 123456
        shares, _ = gen.generate_shares(secret, 5, 3)
        
        recovered = recon.reconstruct_secret(shares[:3])
        assert recovered == secret

    def test_reconstruct_insufficient_shares(self, setup_components):
        gen, _, recon, _, _ = setup_components
        shares, _ = gen.generate_shares(12345, 5, 3)
        
        with pytest.raises(Exception):
            recon.reconstruct_secret(shares[:2])

    def test_reconstruct_invalid_shares(self, setup_components):
        _, _, recon, _, _ = setup_components
        invalid_shares = [(1, 100), (2, 200), (3, 999999)]
        with pytest.raises(Exception):
            recon.reconstruct_secret(invalid_shares)

# Интеграционный тест
def test_full_workflow(crypto_params):
    p, g = crypto_params
    
    # 1. Генерация параметров
    secret = 123456789
    generator = ShareGenerator(p, g)
    
    # 2. Создание долей
    shares, coeffs = generator.generate_shares(secret, 5, 3)
    commitments = generator.generate_commitments(coeffs)
    
    # 3. Проверка долей
    verifier = ShareVerifier(p, g)
    for share in shares:
        assert verifier.verify_share(share, commitments)
    
    # 4. Восстановление
    reconstructor = SecretReconstructor(p)
    recovered_secret = reconstructor.reconstruct_secret(shares[:3])
    
    assert recovered_secret == secret


from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
try:
    long_description = (this_directory / "README.md").read_text(encoding='utf-8')
except Exception as e:
    print(f"Error reading README.md: {e}")
    long_description = "My package description"  # fallback
    
setup(
    name='FELDSHAM',
    version='0.0.4',
    packages=find_packages(),
    install_requires=[
        'sympy>=1.7',
    ],  # Add a comma here
    author='Alexander',
    author_email='lloollfox@mail.ru',
    description='Реализация схемы разделения секрета Фельдмана-Шамира',

    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
     project_urls={
           'Source Repository': 'https://github.com/A-Sharan1/' #replace with your github source
    }
)
